#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: graph.py 2265 2010-02-21 19:16:26Z v13 $

__version__ = "$Id: graph.py 2265 2010-02-21 19:16:26Z v13 $"

from PyQt4 import QtCore
from PyQt4.QtCore import Qt
from PyQt4.QtGui import *

import sys

class WifiColors:
    def __init__(self):
	self.colors=[
	    Qt.green,
	    Qt.cyan,
	    Qt.magenta,
	    Qt.yellow,
	    Qt.red,
	    Qt.darkGreen,
	    Qt.darkCyan,
	    Qt.darkMagenta,
	    Qt.darkYellow,
	    Qt.darkRed,
	    Qt.blue,
	    Qt.darkBlue,
	    Qt.gray,
	    Qt.darkGray
	    ]
	
    def get(self, cnt):
	return(self.colors[cnt%len(self.colors)])

    # Return the number of available colors
    def num(self):
	return(len(self.colors))

class WifiStableColors(WifiColors):
    def __init__(self):
	WifiColors.__init__(self)

	self.used={}	    # Id => Color
	self.usecount={}    # Color=>number of times it was used
	self.restart()

    # Start a new color-usage cycle
    def restart(self):
	self.oldused=self.used
	self.used={}
	self.usecount={}
	for i in self.colors:
	    self.usecount[i]=0

    # Return the next-best unused (or most seldom used) color
    def getNext(self):
	min_=1000
	ret=None
	for i in self.colors:
	    try:
		self.oldused.values().index(i)
		bonus=1
	    except:
		bonus=0

	    if (self.usecount[i]+bonus)<min_:
		ret=i
		min_=self.usecount[i]+bonus

	#print "col:", ret, min_
	return(ret)

    # Get a color for an id
    def get(self, id):
	if self.used.has_key(id):
	    ret=self.used[id]
	    #print "found:", id, ret
	elif self.oldused.has_key(id):
	    ret=self.oldused[id]
	    self.used[id]=ret
	    #print "re-used:", id, ret
	else:
	    ret=self.getNext()
	    self.used[id]=ret
	    #print "new:", id, ret

	self.usecount[ret]+=1

	return(ret)

class BaseGraph(QWidget):
    def __init__(self):
	QWidget.__init__(self)
	
	self.padding_top=20	# This is fixed
	self.space_bottom=80	# This too

	self.width=780	    # Total width - This changes with resizes

	self.bar_x=80
	self.bar_y=350

	self.height=self.bar_y - self.padding_top

	self.colors=WifiStableColors()
    # End of __init__()

    def resizeEvent(self, event):
	sz=event.size()
	w=sz.width()
	h=sz.height()

	self.height=h
	self.width=max(w, 0)
	self.bar_y=max(h - self.space_bottom, 0)
# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

