#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: graphtime.py 2265 2010-02-21 19:16:26Z v13 $

__version__ = "$Id: graphtime.py 2265 2010-02-21 19:16:26Z v13 $"

from PyQt4 import QtCore
from PyQt4.QtCore import Qt
from PyQt4.QtGui import *

import sys, data

from graph import *

class GraphTime(BaseGraph):
    def __init__(self):
	BaseGraph.__init__(self)

	self.dt={}
	
	self.space_bottom=self.padding_top
    # End of __init__()

    def parseData(self):
	dt=data.get()
	#cnt=0
	self.dt={}
	for idx in xrange(data.getMaxlen()):
	    if idx >= len(dt):
		continue

	    # Pass 1: Get the data
	    dt1=dt[idx]
	    for i in dt1:
		d0=dt1[i]
		ssid=d0['essid']
		quality=d0['quality']['quality']
		if not self.dt.has_key(ssid):
		    self.dt[ssid]={'dt':[]}

		while len(self.dt[ssid]['dt'])<idx:
		    self.dt[ssid]['dt'].append(None)

		# If there are multiple APs for one ESSID then display the best
		# quality
		if len(self.dt[ssid]['dt'])==idx+1:
		    oldq=self.dt[ssid]['dt'][idx]
		    self.dt[ssid]['dt'][idx]=max(quality, oldq)
		elif len(self.dt[ssid]['dt'])<=idx:
		    self.dt[ssid]['dt'].append(quality)
		else:
		    print "WTF?", idx, len(self.dt[ssid]['dt'])

	# Pass 2: Allocate colors
	self.colors.restart()
	for i in self.dt:
	    col=self.colors.get(i)
	    self.dt[i]['col']=col
	    #print i, col

	#print
	#print "self.dt:", len(self.dt[i]['dt'])
	#print self.dt
    # End of parseData()

    def paintEvent(self, event):
	axes=QPen(Qt.white, 2, Qt.SolidLine)
	axestext=QPen(Qt.darkGray, 2, Qt.SolidLine)

	#axesdash=QPen(QColor(0x66, 0x66, 0x66), 2, Qt.DashLine)
	axesdash=QPen(QColor(0x66, 0x66, 0x66), 2, Qt.DotLine)

	colbg=QColor(0x33, 0x33, 0x33)

	painter=QPainter(self)
	font0=QFont(painter.font())

	font_ssid=QFont(font0)
	font_ssid.setPointSizeF(12)
	font_ssid_m=QFontMetrics(font_ssid)

	font_xaxis1=QFont(font0)
	font_xaxis1.setPointSizeF(12)
	font_xaxis1_m=QFontMetrics(font_xaxis1)
	font_yaxis1=font_xaxis1
	font_yaxis1_m=font_xaxis1_m

	font_xaxis2=QFont(font0)
	font_xaxis2.setPointSizeF(16)
	font_xaxis2_m=QFontMetrics(font_xaxis2)
	#font_xaxis2.setWeight(font_xaxis2.Bold)
	font_yaxis2=font_xaxis2
	font_yaxis2_m=font_xaxis2_m

	painter.setRenderHints(painter.Antialiasing | \
	    painter.TextAntialiasing | \
	    painter.HighQualityAntialiasing
	    )

	# -----------------------------------
	# Outer rectangle and background

	outerRect=QtCore.QRect(self.bar_x-1, self.padding_top,
		self.width - self.bar_x - 1, self.bar_y - self.padding_top)
	painter.fillRect(outerRect, QBrush(colbg))

	painter.setPen(axes)
	painter.drawRect(outerRect)

	#painter.setBackgroundMode(Qt.TransparentMode)
	painter.setPen(axes)

	# ----------------------------------
	# Y labels
	graphheight=self.height - self.padding_top - self.space_bottom
	last_y=-1000
	painter.setFont(font_yaxis1)
	t_h=font_yaxis1_m.height()
	for i in xrange(0, 101, 10):
	    dy=1.0 * graphheight * i/100
	    y=self.padding_top + graphheight - dy
	    painter.setPen(axes)
	    painter.drawLine(self.bar_x-5, y, self.bar_x+5, y)

	    if abs(y - last_y) > 40:
		painter.setPen(axestext)
		font=QFont(font0)
		font.setPointSizeF(11)
		painter.setFont(font)
		painter.drawText(
		    QtCore.QRect(0, y-(t_h/2), self.bar_x-10, t_h),
		    Qt.AlignRight,
		    "%d%%" % i
		    )
		last_y=y
	    if i==0 or i==100:
		continue
	    painter.setPen(axesdash)
	    painter.drawLine(self.bar_x+5, y, self.width-2, y)
	painter.setFont(font0)

	# Ouch. Rotated text is hard!
	painter.setPen(axestext)
	painter.setFont(font_yaxis2)
	painter.rotate(-90)
	painter.drawText(
	    QtCore.QRect(- self.bar_y, 0, self.bar_y - self.padding_top,
		font_yaxis2_m.height()),
	    Qt.AlignCenter,
	    "Quality"
	    )
	painter.rotate(90)

	# --------------------------------
	# Do data

	self.parseData()

	steps=data.getMaxlen()

	text=[]	    # Hold here ssids that were labeled

	pens={}
	for step in xrange(steps-1):
	    wx=self.width - self.bar_x - 5

	    step0=step
	    step1=step+1
	    x0=self.bar_x + (step0 * wx / (steps-1))+1
	    x1=self.bar_x + (step1 * wx / (steps-1))+1

	    for i in self.dt:
		d0=self.dt[i]
		try:
		    y0=d0['dt'][step0]
		except:
		    y0=None

		try:
		    y1=d0['dt'][step1]
		except:
		    y1=None

		if y0==None or y1==None:
		    continue

		y0=graphheight - (1.0 * graphheight * y0 / 100)
		y1=graphheight - (1.0 * graphheight * y1 / 100)

		y0+=self.padding_top
		y1+=self.padding_top

		if not pens.has_key(d0['col']):
		    pen=QPen(d0['col'], 2, Qt.SolidLine)
		    pens[d0['col']]=pen
		else:
		    pen=pens[d0['col']]
		painter.setPen(pen)
		painter.drawLine(x0, y0, x1, y1)

		try:
		    text.index(i)
		except:
		    text.append(i)
		    painter.setFont(font_ssid)
		    painter.drawText(
			QtCore.QRect(x0, y0 - font_ssid_m.height() - 2,
			    100, font_ssid_m.height()),
			Qt.AlignLeft,
			"%s" % i
			)


# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

