#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: wifilist.py 2265 2010-02-21 19:16:26Z v13 $

__version__ = "$Id: wifilist.py 2265 2010-02-21 19:16:26Z v13 $"

from PyQt4 import QtCore
from PyQt4.QtCore import Qt
from PyQt4.QtGui import *
from PyQt4.QtCore import *

import math
import sys

import data
import channels

class WifiListModel(QAbstractListModel):
    def __init__(self, *args, **kwargs):
	QAbstractListModel.__init__(self, *args, **kwargs)
	self.olddata=None
	self.data=None
	self._sort_dt=None  # Used for the custom-sorting

	self.update_data()


    def data(self, index, role):
	if index.isValid() and role==Qt.DisplayRole:
	    idx=index.row()
	    key=self.data['items'][idx]
	    ret=self.data['data'][key]
	    return(QVariant(ret))
	return(QVariant())

    # Custom sorting function
    #
    # Rationale:
    #	Display first the good-quality signals
    #	Try not to change the location of each ssid by using 5-steps o
    #	sorting (quality/20) and use essid-name sorting inside each step
    #	This way, low-strength signals will not annoy the user
    def sort_cmp(self, a, b):
	dt=self._sort_dt
	dt1=int(dt[a]['quality'] / 20)
	dt2=int(dt[b]['quality'] / 20)

	if dt1!=dt2:
	    ret=dt2-dt1
	else:
	    s1=dt[a]['essid']
	    s2=dt[b]['essid']
	    if s1<s2:
		ret=-1
	    elif s1>s2:
		ret=1
	    else:
		ret=0

#	print "srt:", ret
	return(ret)

    def update_data(self):
	#print "update_data"
	self.olddata=self.data
	self.data={}

	dt0=data.getone()
	dt={}
	for i in dt0:
	    d=dt0[i]
	    if not dt.has_key(d['essid']):
		dt[d['essid']]={
		    'essid':	d['essid'],
		    'aps':	{},
		    'quality':	0,
		    }
	    dt[d['essid']]['aps'][d['bssid']]={
		'freq':	    d['freq'],
		'quality':  d['quality'],
		}
	    if dt[d['essid']]['quality'] < d['quality']['quality']:
		dt[d['essid']]['quality']=d['quality']['quality']

	k=dt.keys()
	self._sort_dt=dt
	k.sort(cmp=self.sort_cmp)

	self.data['items']=k
	self.data['data']=dt

    def update(self):
	#print "update"
	self.update_data()

	# Perhaps optimize this some time
	self.reset()
	return
	
    def rowCount(self, parent):
	ret=len(self.data['items'])
	return(ret)

class WifiListQualityOld(QProgressBar):
    def __init__(self, *args, **kwargs):
	QProgressBar.__init__(self, *args, **kwargs)

	self.setSizePolicy(QSizePolicy.Preferred,
	    QSizePolicy.Minimum)

	self.setMinimumHeight(72)
	self.setMinimumWidth(150)

class WifiListQuality(QWidget):
    """
    A Widget to display the quality of a link
    """
    def __init__(self, *args, **kwargs):
	QWidget.__init__(self, *args, **kwargs)

	self.setSizePolicy(QSizePolicy.Preferred,
	    QSizePolicy.Minimum)

	self.setMinimumHeight(32)
	self.setMinimumWidth(150)

	self.steps=10

	self._value=2

    def paintEvent(self, event):
	painter=QPainter(self)

	painter.setRenderHints(painter.Antialiasing | \
	    painter.TextAntialiasing | \
	    painter.HighQualityAntialiasing
	    )

	font0=QFont(painter.font())

	font_perc=QFont(font0)
	#font_perc.setPointSizeF(14)
	font_perc.setPixelSize(self.height() * 2 / 3)
	font_perc_m=QFontMetrics(font_perc)

	col_perc=Qt.white
	pen_perc=QPen(col_perc)

	x0=0
	y0=0

	# Margin between graph and text
	txt_margin=4

	# Space required for text plus some margin
	txt_space = font_perc_m.width("IXXXX") + txt_margin

	# The width of the bar plus its borders
	bar_width=self.width() - txt_space

	# Space between each "cell"
	spacing=2
	# Border-line width
	line_width=1
	# Space between cells and border
	cell_padding_x=3
	cell_padding_y=3
	# Step between each cell: 
	x_step = math.floor(
	    (bar_width - (2*cell_padding_x) - ((self.steps-1) * spacing) - \
		(line_width * 2)) / self.steps) + spacing

	# Since we did some calculations and we need integer results,
	# recalculate the width of the bar by multuplying the cell size
	# with the number of cells
	width = (x_step * self.steps) + (line_width*2) + (spacing*2)

	# Draw the text
	painter.setPen(pen_perc)
	painter.setFont(font_perc)
	txt_rect = QRect(self.width() - txt_space, 2, txt_space,
	    font_perc_m.height())
	painter.drawText(txt_rect, Qt.AlignRight, "%s%%" % (self._value))

	# Draw the graph
	w=x_step - spacing
	h=self.height() - (cell_padding_y*2) - (line_width*2)

	for step in xrange(self.steps):
	    x=x0 + (step*x_step) + line_width + cell_padding_x
	    y=y0 + cell_padding_y + line_width

	    rect=QRect(x, y, w, h)

	    # Fill color
	    colfill=QColor(
		0x00,
		0x44 + 0xbb * ((1.0*step / self.steps)**2),
		0x00)

	    # Fill color for "empty" cells
	    colfill0=QColor(0x00, 0x22, 0x00)

	    # Brushes for cells: One for the filled cell, another for the
	    # non-filled cell
	    br=QBrush(QColor(colfill))
	    br0=QBrush(QColor(colfill0))

	    t1=step*100/self.steps
	    # Leave this "<". "<=" doesn't look beautifull with
	    # (e.g) ten steps and 90%
	    if t1 < self._value:
		painter.fillRect(rect, br)
	    else:
		painter.fillRect(rect, br0)

	# Draw the outline of cells
	# Outline color
	colol=QColor(0x33, 0x99, 0x33)
	painter.setPen(QPen(colol))

	rect=QRect(x0,y0,width, self.height())
	painter.drawRoundedRect(rect, 4, 5)

    def setValue(self, value):
	self._value=value

    def sizeHint(self):
	ret=QSize(180, 32)
	return(ret)

class WifiListWidgetAP(QWidget):
    def __init__(self, *args, **kwargs):
	QWidget.__init__(self, *args, **kwargs)

	self.l0=QHBoxLayout(self)

	self.l0.addSpacing(20)

	self.setObjectName("ap")

	# Calculate the font size (80% of the original)
	self.lBssid=QLabel(self)
	self.lBssid.setObjectName("bssid")
	font=QFont(self.lBssid.font())
	font.setPointSizeF(font.pointSizeF()*0.8)
	self.lBssid.setFont(font)
	self.l0.addWidget(self.lBssid)

	self.l0.addSpacing(20)

	self.lChannel=QLabel(self)
	self.lChannel.setObjectName("channel")
	self.lChannel.setFont(font)
	self.l0.addWidget(self.lChannel)

	spacer=QSpacerItem(40, 20, QSizePolicy.Expanding)
	self.l0.addItem(spacer)

	self.pQuality=WifiListQuality(self)
	self.l0.addWidget(self.pQuality)

    def setData(self, dt):
	"""
	dt:
	- bssid
	- freq
	- quality
	"""
	self.lBssid.setText(dt['bssid'])
	self.lChannel.setText("Ch %d - %s MHz" % (dt['channel'],  dt['freq']))
	self.pQuality.setValue(dt['quality'])


class WifiListWidget(QFrame):
    """
    Interesting items:

    - lEssid:	    The ESSID label
    - pQuality:	    The progress-bar for quality
    """
    def __init__(self, apnum, *args, **kwargs):
	QFrame.__init__(self, *args, **kwargs)
	
	# A list to hold the widgets for each AP
	self.apw=[]

	self.create(apnum)

    def create(self, apnum):
	"""
	apnum: The number of detected access points
	"""
	self.l0=QVBoxLayout(self)
	self.l1=QHBoxLayout()

	self.lEssid=QLabel(self)
	self.l1.addWidget(self.lEssid)

	spacer=QSpacerItem(40, 20, QSizePolicy.Expanding)
	self.l1.addItem(spacer)

	self.pQuality=WifiListQuality(self)
	self.l1.addWidget(self.pQuality)

	self.l0.addLayout(self.l1)

	for i in xrange(apnum):
	    w=WifiListWidgetAP(self)
	    self.l0.addWidget(w)
	    self.apw.append(w)

#	self.setBackgroundRole(QPalette.Window)

	self.setStyleSheet("""
	QLabel {
	    color: rgb(255, 192, 64);
	    }
	#ap QLabel {
	    color: rgb(128,192,255);
	    }
	""")
    def setData(self, dt):
	"""
	dt:
	- essid
	- quality
	- aps:
	  - whatever WifiListWidgetAP.setData() needs
	"""

	self.lEssid.setText(dt['essid'])
	self.pQuality.setValue(dt['quality'])

	t=dt['aps'].keys()
	t.sort()
	for i in xrange(len(t)):
	    t2=dt['aps'][t[i]]
	    self.apw[i].setData(t2)

class WifiListDelegate(QStyledItemDelegate):
    def __init__(self, *args, **kwargs):
	QStyledItemDelegate.__init__(self, *args, **kwargs)

    def _get_data(self, index):
	dt0=index.data(Qt.DisplayRole).toPyObject()
	#print dt

	dt={
	    'essid':	dt0[QString('essid')],
	    'quality':	0,
	    'aps':	{},
	    }

	_maxquality=0
	for i in dt0[QString('aps')]:
	    dt1=dt0[QString('aps')][i]
	    #print "dt1:", dt1

	    _bssid=i
	    _freq=dt1[QString('freq')][QString('m')]
	    _quality=dt1[QString('quality')][QString('quality')]

	    if _quality > _maxquality:
		_maxquality=_quality

	    dt['aps'][_bssid]={
		'bssid':    _bssid,
		'freq':	    _freq,
		'quality':  _quality,
		'channel':  channels.freqToChan(_freq),
		}

	dt['quality']=_maxquality

	return(dt)

    def paint(self, painter, option, index):
	#print "paint", index.row()
	painter.save()

	cBg=Qt.gray
	brBg=QBrush(cBg)

	dt=self._get_data(index)

	cLabel1=Qt.green

	dev=painter.device()
	painter.restore()

	w=WifiListWidget(len(dt['aps']))
	w.resize(option.rect.size())
	w.setData(dt)

	rect=QRect(option.rect)
	rect2=w.rect()

	w.render(painter, rect.topLeft(), QRegion(rect2))
	painter.save()

	painter.restore()

    def sizeHint(self, option, index):
	dt=self._get_data(index)
    
	# This is called A-LOT. Optimize it.
#	print "sh0:"
	w=WifiListWidget(len(dt['aps']))
	w.deleteLater()
	ret=w.sizeHint()
	return(ret)

class WifiList(QListView):
    def __init__(self, *args, **kwargs):
	QListView.__init__(self, *args, **kwargs)

	self.model=WifiListModel(self)
	delegate=WifiListDelegate(self)
	self.setModel(self.model)
	self.setItemDelegate(delegate)

	# TODO: Fix this?
#	self.setProperty("FingerScrollable", True)

    def update(self):
	self.model.update()
# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

