#!/usr/bin/env python
# coding=UTF-8
#
# Copyright (C) 2010 Stefanos Harhalakis
#
# This file is part of wifieye.
#
# wifieye is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# wifieye is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with wifieye.  If not, see <http://www.gnu.org/licenses/>.
#
# $Id: win.py 2265 2010-02-21 19:16:26Z v13 $

__version__ = "$Id: win.py 2265 2010-02-21 19:16:26Z v13 $"

from PyQt4 import QtGui, QtCore
from PyQt4.QtCore import SIGNAL, Qt
from PyQt4.QtGui import *

import sys

import graphchannels, graphtime, wifi, data, wifilist
#import portrait

win=None

whatOptions={
	    "channels":	    "Channels",
	    "time":	    "Time Graph",
	    "list":	    "Details",
    }
whatOptionsList=['channels', 'time', 'list']

# A Model for the available views
class WhatOptionsModel(QtCore.QAbstractListModel):
    def __init__(self, *args, **kwargs):
	QtCore.QAbstractListModel.__init__(self, *args, **kwargs)

    def data(self, index, role):
	global whatOptions, whatOptionsList

	if not index.isValid():
	    return(QtCore.QVariant())

	if index.row() >= len(whatOptions):
	    return(QtCore.QVariant())

	if role==Qt.DisplayRole:
	    id=whatOptionsList[index.row()]
	    return(whatOptions[id])

	return(QtCore.QVariant())

    def rowCount(self, parent):
	global whatOptionsList

	return(len(whatOptionsList))

    def getIdByIdx(self, idx):
	global whatOptionsList

	return(whatOptionsList[idx])

    def getIdxById(self, id):
	global whatOptionsList

	return(whatOptionsList.index(id))

# A dialog to select the desired view
class WhatPickerDialog(QDialog):
    def __init__(self, *args, **kwargs):
	QDialog.__init__(self, *args, **kwargs)

	self.recreate()
	self.retranslateUi()

	self.selection=None	# The string representation of the current
				# selections ("channels", etc..)

	self.connect(self.lst, SIGNAL("clicked(const QModelIndex&)"),
	    self.slotListClicked, Qt.QueuedConnection)

    def recreate(self):
	self.l0=QHBoxLayout(self)

	self.lst=QListView(self)
	self.model=WhatOptionsModel(self.lst)
	self.lst.setModel(self.model)
	self.lst.setProperty("FingerScrollable", True)
	
	self.l0.addWidget(self.lst)

    def retranslateUi(self):
	self.setWindowTitle("Select view")

    def slotListClicked(self, index):
	idx=index.row()
	self.selection=self.model.getIdByIdx(idx)
	self.accept()

    # Set the current selected item.
    # sel is the string id ("channels", etc)
    def setSelection(self, sel):
	# Store this
	self.selection=sel
	# Get the index from the string
	idx=self.model.getIdxById(sel)
	# Convert it to QModelIndex
	idx2=self.model.index(idx, 0)
	# Set the selection
	self.lst.setCurrentIndex(idx2)

    def getSelection(self):
	return(self.selection)

# Common widgets between all views
class Ui_Common:
    def __init__(self, what):
	self.what=what
	self.centralwidget=None
	self.slotButtonWhatDialog=None
	self.buttonWhatSelection=what
	self.w=None	    # The widget that should be shown

    # @param what   The current view: "channels", etc
    def getCommonButtons(self):
	""" Return common buttons """
	global whatOptions, whatOptionsList

	ret=[]

	self.buttonWhat=QtGui.QPushButton()
	ret.append(self.buttonWhat)

	txt=whatOptions[self.what]
	#self.buttonWhat.setText(txt)
	self.buttonWhat.setText("Select view")

	QtCore.QObject.connect(self.buttonWhat, SIGNAL('clicked()'),
	    self.slotButtonWhat)

	return(ret)

    def slotButtonWhat(self):
	# Create the dialog (or reuse it)
	if self.slotButtonWhatDialog==None:
	    d=WhatPickerDialog()
	    d.connect(d, SIGNAL('finished(int)'), self.slotButtonWhatFinished)
	else:
	    d=self.slotButtonWhatDialog

	# Set the correct current value
	d.setSelection(self.what)

	d.show()
	self.slotButtonWhatDialog=d

    def slotButtonWhatFinished(self, result):
	d=self.slotButtonWhatDialog

	# Get the selected item
	self.buttonWhatSelection=d.getSelection()

	#print "sel:", self.buttonWhatSelection

	# Emit a signal from the button
	self.buttonWhat.emit(QtCore.SIGNAL("selected"))

class Ui_Channels(Ui_Common):
    def __init__(self):
	Ui_Common.__init__(self, 'channels')

    def setupUi(self, w):
	self.centralwidget=QtGui.QWidget(w)
	self.centralwidget.setObjectName("centralwidget")

	self.l0=QtGui.QVBoxLayout(self.centralwidget)

	self.graph=graphchannels.GraphChannels()
	self.graph.setObjectName("graph")
	self.l0.addWidget(self.graph)

	# The buttons
	self.l1=QtGui.QHBoxLayout()

	# Common buttons first
	cmn=self.getCommonButtons()
	for i in cmn:
	    self.l1.addWidget(i)

	self.buttonRefresh=QtGui.QPushButton()
	self.l1.addWidget(self.buttonRefresh)

	self.buttonAutoRefresh=QtGui.QPushButton()
	self.buttonAutoRefresh.setCheckable(True)
	self.l1.addWidget(self.buttonAutoRefresh)

	spacer=QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding,
	    QtGui.QSizePolicy.Minimum)
	self.l1.addItem(spacer)
	self.l0.addLayout(self.l1)

	self.w=self.centralwidget

    def retranslateUi(self):
	self.buttonRefresh.setText("Refresh")
	self.buttonAutoRefresh.setText("Auto Refresh")

class Ui_Time(Ui_Common):
    def __init__(self):
	Ui_Common.__init__(self, 'time')

    def setupUi(self, w):
	self.centralwidget=QtGui.QWidget(w)
	self.centralwidget.setObjectName("centralwidget")

	self.l0=QtGui.QVBoxLayout(self.centralwidget)

	self.graph=graphtime.GraphTime()
	self.graph.setObjectName("graph")
	self.l0.addWidget(self.graph)

	# The buttons
	self.l1=QtGui.QHBoxLayout()

	# Common buttons first
	cmn=self.getCommonButtons()
	for i in cmn:
	    self.l1.addWidget(i)

	self.buttonClear=QtGui.QPushButton()
	self.l1.addWidget(self.buttonClear)

	self.buttonPause=QtGui.QPushButton()
	self.buttonPause.setCheckable(True)
	self.l1.addWidget(self.buttonPause)

	spacer=QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding,
	    QtGui.QSizePolicy.Minimum)
	self.l1.addItem(spacer)
	self.l0.addLayout(self.l1)

	self.w=self.centralwidget

    def retranslateUi(self):
	self.buttonClear.setText("Clear")
	self.buttonPause.setText("Pause")

class Ui_List(Ui_Common):
    def __init__(self):
	Ui_Common.__init__(self, 'list')

    def setupUi(self, w):
	self.centralwidget=QtGui.QWidget(w)

	self.l0=QtGui.QVBoxLayout(self.centralwidget)

	self.lst=wifilist.WifiList(self.centralwidget)
	self.l0.addWidget(self.lst)

	self.l1=QtGui.QHBoxLayout()
	cmn=self.getCommonButtons()
	for i in cmn:
	    self.l1.addWidget(i)

	self.buttonRefresh=QtGui.QPushButton()
	self.l1.addWidget(self.buttonRefresh)

	self.buttonAutoRefresh=QtGui.QPushButton()
	self.buttonAutoRefresh.setCheckable(True)
	self.l1.addWidget(self.buttonAutoRefresh)

	spacer=QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding,
	    QtGui.QSizePolicy.Minimum)
	self.l1.addItem(spacer)


	self.l0.addLayout(self.l1)

	self.w=self.centralwidget

    def retranslateUi(self):
	self.buttonRefresh.setText("Refresh")
	self.buttonAutoRefresh.setText("Auto Refresh")

class MainWin(QtGui.QMainWindow):
    def __init__(self, *args, **kwargs):
	QtGui.QMainWindow.__init__(self, *args, **kwargs)

	self.ui={}
	self.what=None

	# An auto-refresh timer and its callback
	self.artimer=None
	self.artimer_cb=None

#	self.chtimer=None
#	self.tmtimer=None

	self.what=None

	self.setupUi()
	self.retranslateUi()
	self.selectUi("channels")
	#self.selectUi("list")

    def setupUiChannels(self):
	ui=Ui_Channels()
	ui.setupUi(self.centralwidget)

	self.connect(ui.buttonRefresh, SIGNAL('clicked()'),
		self.slotChRefresh)
	self.connect(ui.buttonAutoRefresh, SIGNAL('toggled(bool)'),
		self.slotChAutoRefresh)

	self.ui['channels']=ui

    def setupUiTime(self):
	ui=Ui_Time()
	ui.setupUi(self.centralwidget)

	self.connect(ui.buttonClear, SIGNAL('clicked()'),
		self.slotTmClear)
	self.connect(ui.buttonPause, SIGNAL('toggled(bool)'),
		self.slotTmPause)

	self.ui['time']=ui

    def setupUiList(self):
	ui=Ui_List()
	ui.setupUi(self.centralwidget)

	self.connect(ui.buttonRefresh, SIGNAL('clicked()'),
		self.slotLstRefresh)
	self.connect(ui.buttonAutoRefresh, SIGNAL('toggled(bool)'),
		self.slotLstAutoRefresh)

	self.ui['list']=ui

    def retranslateUi(self):
	self.setWindowTitle("WifiEye")
	self.ui['channels'].retranslateUi()
	self.ui['time'].retranslateUi()
	self.ui['list'].retranslateUi()

    def setupUi(self):
	sw=QStackedWidget(self)
	self.centralwidget=sw
	self.setCentralWidget(sw)

	self.setupUiChannels()
	self.setupUiTime()
	self.setupUiList()

	sw.addWidget(self.ui['channels'].w)
	sw.addWidget(self.ui['time'].w)
	sw.addWidget(self.ui['list'].w)

	# Do common signals
	for i in self.ui:
	    self.connect(self.ui[i].buttonWhat, SIGNAL('selected'),
		self.slotCommonWhat, Qt.QueuedConnection)

    def selectUi(self, what):
	if self.what==what:
	    return

	self.what=what
	sw=self.centralwidget

	# Stop all timers first
	self.stopTimers()

	if what=='channels':
	    #self.stopTmTimer()
	    sw.setCurrentWidget(self.ui['channels'].w)
	    self.slotChRefresh()
	elif what=='time':
	    #self.stopChTimer()
	    sw.setCurrentWidget(self.ui['time'].w)
	    self.startTmTimer()
	elif what=='list':
	    sw.setCurrentWidget(self.ui['list'].w)
	    self.slotLstRefresh()
	else:
	    print "WTF? Bad what:", what

    # Internal function:
    # Start the auto-refresh timer and connect its timeout to the
    # specified callback function
    def startArTimer(self, cb):
	# If we don't need to change something then return
	if self.artimer!=None:
	    if self.artimer_cb==cb:
		return

	# Else stop it and recreate it
	self.stopArTimer()

	self.artimer=QtCore.QTimer(self)
	self.connect(self.artimer, SIGNAL('timeout()'), cb)
	self.artimer.start(1300)
	self.artimer_cb=cb

    def stopArTimer(self):
	if self.artimer==None:
	    return
	self.artimer.stop()
	self.artimer.deleteLater()
	self.artimer=None
	self.artimer_cb=None

    def startChTimer(self):
	self.startArTimer(self.slotChTimerAutorefresh)

#    def stopChTimer(self):
#	self.stopArTimer()

    def startTmTimer(self):
	self.startArTimer(self.slotTmTimerGetData)
	
    def stopTmTimer(self):
	self.stopArTimer()

    def startLstTimer(self):
	self.startArTimer(self.slotLstTimerAutorefresh)

#    def stopLstTimer(self):
#	self.stopArTimer()

    def stopTimers(self):
	self.stopArTimer()

    # -------------------------------
    # Slots:
    # slotCommon are Common slots
    # slotCh* are for Channel view
    # slotTm* are for Time view

    def slotCommonWhat(self):
	ui=self.ui[self.what]
	w=ui.buttonWhatSelection
	if w==self.what:
	    return

	self.selectUi(w)

    # -------------------------------

    # Common slot for Channel and List view
    # @param cb	    The callback for the timer
    def slotChLstRefresh(self, what, cb):
	ui=self.ui[what]
	if ui.buttonAutoRefresh.isChecked():
	    wifi.scanAsync(cb)
	else:
	    # Only disable the button when scanAsync() succeeds
	    # Else we may stay with a disabled button
	    if wifi.scanAsync(cb):
		ui.buttonRefresh.setEnabled(False)

    # Common slot for Channel and List view
    # @param what	channels/list
    # @pram startTimer	A function to call to start the timer
    def slotChLstAutoRefresh(self, what, startTimer, checked):
	ui=self.ui[what]
	if checked:
	    ui.buttonRefresh.setEnabled(False)
	    startTimer()
#	    if self.artimer==None:
#		self.startArTimer(cb)
	else:
	    ui.buttonRefresh.setEnabled(True)
	    self.stopArTimer()

    # Common slot for Channel and List view
    def slotChLstScanAsync(self, what, dt):
	ui=self.ui[what]
	if self.what!=what:
	    return
	if not ui.buttonAutoRefresh.isChecked():
	    ui.buttonRefresh.setEnabled(True)
	data.setone(dt)

    def slotChTimerAutorefresh(self):
	if self.what!='channels':
	    return
	self.slotChRefresh()

    # Button "refresh" pressed
    def slotChRefresh(self):
	self.slotChLstRefresh('channels', self.slotChScanAsync)

    # Button "auto-refresh" pressed
    def slotChAutoRefresh(self, checked):
	self.slotChLstAutoRefresh('channels', self.startChTimer, checked)

    # This is called asynchronously by the wifi scanner
    def slotChScanAsync(self, dt):
	self.slotChLstScanAsync('channels', dt)
	#self.repaint()
	ui=self.ui['channels']
	ui.graph.repaint()

    # This is called when auto-refresh is enabled by the auto-refresh timer
    def slotChTimerAutorefresh(self):
	if self.what!='channels':
	    return
	self.slotChRefresh()

    # -------------------------------

    def slotLstRefresh(self):
	self.slotChLstRefresh('list', self.slotLstScanAsync)

    def slotLstAutoRefresh(self, checked):
	self.slotChLstAutoRefresh('list', self.startLstTimer, checked)

    def slotLstScanAsync(self, dt):
	self.slotChLstScanAsync('list', dt)
	ui=self.ui['list']
	ui.lst.update()

    def slotLstTimerAutorefresh(self):
	if self.what!='list':
	    return
	self.slotLstRefresh()

    # -------------------------------
    def slotTmTimerGetData(self):
	if self.what!='time':
	    return
	dt=wifi.scanAsync(self.slotTmScanAsync)

    def slotTmClear(self):
	data.clear()
	self.repaint()

    def slotTmScanAsync(self, dt):
	if self.what!='time':
	    return
	data.add(dt)
	#self.repaint()
	self.ui['time'].graph.repaint()

    def slotTmPause(self, checked):
	if checked:
	    self.stopTmTimer()
	else:
	    self.startTmTimer()


def init():
    global app, win, por
    app=QtGui.QApplication(sys.argv)
    #app.setAttribute(Qt.WA_Maemo5ForceLandscapeOrientation, True);
#    win0=QtGui.QMainWindow()
    win=MainWin()
    #por=portrait.FremantleRotation("wifieye", win, '1.0')

    #win.setupUi()

def doit(dt):
    global app, win

    win.show()
    ret=app.exec_()
    wifi.finish()

    sys.exit(ret)

# vim: set ts=8 sts=4 sw=4 noet formatoptions=r ai nocindent:

